
# Source needed utility functions.
source(here::here("code", "seurat-utils.R"))

#' Find marker genes using Seurat
#'
#' This wraps the `Seurat::FindAllMarkers` functions.
#'
#' @param sce A SingleCellExperiment object,
#' @param pars A list of parameters passed to the function. For Seurat this has
#'   the following components:
#'   * `test.use`: The test to use.
#'
#' @details The `FindAllMarkers` function compares a specified cluster to *all*
#'   other clusters in the data.
#'
#' @return A list containing the time of the computation and results of the
#'  method.
#'
run_seurat <- function(sce, pars) {

  seurat_data <- as_seurat_data(sce)

  times <- system.time({
    raw_result <- Seurat::FindAllMarkers(
      seurat_data,
      test.use = pars$test.use
    )
  })

  raw_result <- tibble::as_tibble(raw_result)
  if (!pars$test.use == "roc") {
    result <- dplyr::select(
      raw_result,
      p_value = p_val,
      p_value_adj = p_val_adj,
      cluster,
      log_fc = avg_log2FC,
      gene = gene
    )
    result <- dplyr::mutate(
      result,
      raw_statistic = 0,
      scaled_statistic = 0
    )
  } else {
    result <- dplyr::select(
      raw_result,
      log_fc = avg_log2FC,
      raw_statistic = power,
      gene,
      cluster
    )
    result <- dplyr::mutate(
      result,
      scaled_statistic = 0,
      p_value = 0,
      p_value_adj = 0
    )
  }

  result <- tibble::as_tibble(result)

  validate_mgs_result(result)

  # Get the elapsed time.
  time <- times[[3]]

  list(time = time, result = result, raw_result = raw_result, pars = pars)
}
